float4x4 g_mViewProj;

void VS_RenderShadowMap(
  float4 vPos : POSITION,
  out float4 vPosOut : POSITION,
  out float3 vPixelOut : TEXCOORD0)
{
  // pass vertex position through as usual
  vPosOut = mul(vPos, g_mViewProj);
  // output pixel pos
  vPixelOut=vPosOut.xyz;
}

float4 PS_RenderShadowMap(float3 vPixelPos : TEXCOORD0): COLOR
{
  // write z coordinate (linearized depth) to texture
  return vPixelPos.z;
}


// This technique is used when rendering meshes to the shadowmap
// 
technique RenderShadowMap
{
  pass p0
  {
    // render back faces to hide artifacts
    CullMode = CW;
    VertexShader = compile vs_2_0 VS_RenderShadowMap();
    PixelShader = compile ps_2_0 PS_RenderShadowMap();
  }
}


//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////


float3 g_vLightDir;
float3 g_vLightColor;
float3 g_vAmbient;
float g_fShadowMapSize;
float4x4 g_mShadowMap;

// no filtering in floating point texture
sampler2D g_samShadowMap  =
sampler_state
{
  MinFilter = Point;
  MagFilter = Point;
  MipFilter = None;
  AddressU = Border;
  AddressV = Border;
  BorderColor = 0xFFFFFFFF;
};


void VS_Shadowed(
  in float4 vPos : POSITION,
  in float3 vNormal : NORMAL,
  in float fAmbientIn : TEXCOORD0,
  out float4 vPosOut : POSITION,
  out float4 vShadowTex : TEXCOORD0,
  out float fAmbientOut : TEXCOORD1,
  out float3 vDiffuse : COLOR0)
{
  // pass vertex position through as usual
  vPosOut = mul(vPos, g_mViewProj);
  
  // calculate per vertex lighting
  vDiffuse = g_vLightColor * saturate(dot(-g_vLightDir, vNormal));

  // coordinates for shadowmap
  vShadowTex = mul(vPos, g_mShadowMap);

  // ambient occlusion
  fAmbientOut = saturate(0.5f+fAmbientIn);
}

float4 PS_Shadowed(
  float4 vShadowTex : TEXCOORD0,
  float fAmbientOcclusion : TEXCOORD1,
  float4 vDiffuse : COLOR0) : COLOR
{

  float fTexelSize=1.0f/g_fShadowMapSize;

  // project texture coordinates
  vShadowTex.xy/=vShadowTex.w;
  
  // 2x2 PCF Filtering
  // 
  float fShadow[4];
  fShadow[0] = (vShadowTex.z < tex2D(g_samShadowMap, vShadowTex).r);
  fShadow[1] = (vShadowTex.z < tex2D(g_samShadowMap, vShadowTex + float2(fTexelSize,0)).r);
  fShadow[2] = (vShadowTex.z < tex2D(g_samShadowMap, vShadowTex + float2(0,fTexelSize)).r);
  fShadow[3] = (vShadowTex.z < tex2D(g_samShadowMap, vShadowTex + float2(fTexelSize,fTexelSize)).r);
  
  float2 vLerpFactor = frac(g_fShadowMapSize * vShadowTex);
  float fLightingFactor = lerp(lerp( fShadow[0], fShadow[1], vLerpFactor.x ),
                               lerp( fShadow[2], fShadow[3], vLerpFactor.x ),
                               vLerpFactor.y);

  // multiply diffuse with shadowmap lookup value
  vDiffuse*=fLightingFactor;
  
  // final color
  float4 vColor=1;
  vColor.rgb = saturate(g_vAmbient*fAmbientOcclusion + vDiffuse).rgb;
  return vColor;
}

// This technique is used to render the final shadowed meshes
//
technique Shadowed
{
  pass p0
  {
    // render front faces
    CullMode = CCW;
	  VertexShader = compile vs_2_0 VS_Shadowed();
	  PixelShader = compile ps_2_0 PS_Shadowed();
  }
}